/*
	MARCU DENIS_GEORGE
	332CA
*/

#include "game.h"


/* main */
int main(void)
{	
	// Setup LCD
	lcd_init();
	lcd_contrast(0x40);
	
	// initializez butoanele, buzzerul si bufferul folosit pt lcd
	init_buttoms();
	init_buzzer();
	init_buffer();
	
	// Deschidem cadrul de start al jocului
	openning_program();
	
	// Intergata jocului
	init_game();
	
	player.x = 58;
	
	while (1) {
		lcd_clear();
		init_buffer();
		select_game();
		lives = 3;
	}	
}


/* Initializarea butoanelor */
static void init_buttoms(void)
{
	/* Setez pinul PD0, PD1, PD4 ca intrare. */
	DDRD &= ~B_LEFT;
	DDRD &= ~B_RIGHT;
	DDRD &= ~B_OK;

	/* Activez rezistenta de pull-up pentru pinul PD0, PD1, PD4. */
	PORTD |= B_LEFT;
	PORTD |= B_RIGHT;
	PORTD |= B_OK;
}


/* initializarea buzzer-ului */
static void init_buzzer(void)
{
	/* Setez buzzerul ca iesire ca iesire. */
	DDRD |= BUZZER;

	/* Opresc buzzer-ul. */
	PORTD &= ~BUZZER;
}


/* Redau o nota folosind buzzer-ul. */
static void buzzer_reda(int nota)
{
	int i;

	/* Alternam starea pinului PD5 la fiecare milisecunda. */
	for (i = 0; i < nota; ++i) {
		PORTD ^= BUZZER;

		_delay_ms(1);
	}
}


/* Initializarea bufferul pentru lcd */
static void init_buffer(void)
{
	int i, j;
	for (i = 0; i < BUFF_SIZE_Y; i++)
		for (j = 0; j < BUFF_SIZE_X; j++)
			lcd_buffer[i][j] = 0;
}


/* Functie de setare a unui pixel */
static void set_pixel(uint8_t x, uint8_t y, COLOR color)
{
	uint8_t row = y / 8;
	uint8_t data = lcd_buffer[row][x];
	
	if (color == BLACK) 
		data |= (1 << (y % 8));
	else
		data &= ~(1 << (y % 8));
		

	// salvam noua valoare a datei
    lcd_buffer[row][x] = data;

    lcd_send(0x80 | x, LCD_CMD);
    lcd_send(0x40 | row, LCD_CMD);

    lcd_send(data, LCD_DATA);
}


/* Functie de delay */
static void delay(uint8_t n)
{
    for (uint8_t i = 0; i < n; i++)
        _delay_ms(1000);
}


/* Functie ce afiseaza deschiderea unui program */
static void openning_program(void)
{
	// Afisez date introductive
	lcd_mystr_delay("PM2017", 15, 6);
	buzzer_reda(NOTA);
	delay(10);
		
	lcd_mystr_delay("MARCU", 32, 6);
	buzzer_reda(NOTA);
	
	lcd_mystr_delay("DENIS", 42, 5);
	buzzer_reda(NOTA);
	delay(10);
	
	lcd_mystr_delay("332CA", 59, 6);
	buzzer_reda(NOTA);
	delay(10);
	
	// Afisez bara de loading
	int i;
	int size1 = LCD_X_RES - 1;
	int size2 = LCD_X_RES - 2;
	int size3 = LCD_X_RES - 3;
	for(i = LCD_Y_RES - 1;i >= 0; i--)
    {
		
		set_pixel(size1, i, BLACK);
		set_pixel(size2, i, BLACK);
		set_pixel(size3, i, BLACK);
		
		if (i % 4 == 0)
			delay(5);
    }
	
	delay(1);
	lcd_clear();
}


/* Functie de initializare a jocului */
static void init_game(void)
{
	int y = 37;
	int i, exit;

	// animatie pentru coborarea pe lcd a textului "CAR\nRACE"
	// impreuna cu o masina de curse
	for (i = -62; i < 8; i+=2) {
			// afisez cuvantul CAR/RAce incepand cu o anumita pozite
			
			if ( i > -18)
				lcd_car(i, y, BLACK);
				
			if(i > -35)
				lcd_race(i + 17, y + 4, BLACK);
			
			print_car_reverse(i + 34, 31, BLACK);
			
			delay(2);
			buzzer_reda(NOTA);

			// Stergem ce am afisat pentru a putea afisa la urmatorul pas
			if (i > -18)
				lcd_car(i, y, WHITE);
			
			if (i > -35)
				lcd_race(i + 17, y + 4, WHITE);
				
			print_car_reverse(i + 34, 31, WHITE);
	}
	
	// Afisam imaginea finala (logo-ul jocului)
	i -= 2;
	lcd_car(i, y, BLACK);
	lcd_race(i + 17, y + 4, BLACK);
	print_car_reverse(i + 34, 31, BLACK);
	
	delay(4);
	
	// Asteptare buton OK
	while (1) {
		
		exit = 0;
		
		buzzer_reda(NOTA);
		lcd_mystr("OK", 77, 4);
	
		// verific daca am apasat butonu. Am aici si un
		// delay pentru ca "ok" sa apara intermitent
		for (i = 0; i < 5; i++) {
			if ((PIND & B_OK) == 0) {
				exit = 1;
				break;
			}
			delay(1);
		}
		
		// conditie iesire
		if (exit) break;
		
		// stergem ok-ul
		lcd_clear_area(4, 77, 84);
		lcd_clear_area(3, 77, 84);
		
		// verificam din nou daca s-a apasat butonul ok
		for (i = 0; i < 5; i++) {
			if ((PIND & B_OK) == 0) {
				exit = 1;
				break;
			}
			delay(1);
		}
		
		// conditie iesire
		if (exit) break;
	}
	
	delay(2); 
}


/* Functie ce printeaza o litera mare (C/A/R/E) */
static void print_letter(uint8_t x, uint8_t y, COLOR color, uint8_t index)
{
	int i, j, size;
	
	size = index + 12;
	
	// Avem salvate numerele intr-o matrice si setam
	// pixelii in functie de aceasta
	for (i = index; i < size; i++) {
		for (j = 0; j < 7; j++) {
			if (letter_matrix[i][j] == 1) {
				set_pixel(x + (i % 12), y - j, color);
			}
		}
	}
}


/* Afisez cuvantul CAR */
static void lcd_car(uint8_t x, uint8_t y, COLOR color)
{
	// print C
	print_letter(x, y, color, 0);
	y -= 9;
	
	// print A
	print_letter(x, y, color, 12);	
	y -= 9;
	
	// print R
	print_letter(x, y, color, 24);
	
}


/* Afisez masina cuvantul RACE */
static void lcd_race(uint8_t x, uint8_t y, COLOR color)
{	
	// print R
	print_letter(x, y, color, 24);
	y -= 9;
	
	// print A
	print_letter(x, y, color, 12);	
	y -= 9;
	
	// print C
	print_letter(x, y, color, 0);
	y -= 9;
	
	// print E
	print_letter(x, y, color, 36);
}


/* Printez masina in functie de matricea definita */
static void print_car(int x, int y, COLOR color)
{
	int i, j;
	int new_x, new_y;
	
	for (i = 0; i < 26; i++) {
		for (j = 0; j < 14; j++) {
			if (car_matrix[i][j] == 1) {
				new_x = x + i;
				new_y = y - j;
				
				// verific daca intru in interval pe verticala
				if (new_x >= 0 && new_x < LCD_X_RES) {
					
					set_pixel(new_x, new_y, color);
				}
				
			}
		}
	}

}


/* Printez masina invers in functie de matricea definita */
static void print_car_reverse(int x, int y, COLOR color)
{
	int i, j;
	int new_x, new_y;
	
	for (i = 0; i < 26; i++) {
		for (j = 0; j < 14; j++) {
			if (car_matrix[i][j] == 1) {
				new_x = x + (25 - i);
				new_y = y - j;
				
				// verific daca intru in interval pe verticala
				if (new_x >= 0 && new_x < LCD_X_RES) {
					
					set_pixel(new_x, new_y, color);
				}
			}
		}
	}

}


/* Afisam linie punctata in functie de cazuri (am 3 cazuri de linie
 de inceput: 4 BLACK, 2 BLACK, 2 WHITE, 4 WHITE
*/
static void print_line(int x, int y, COLOR color)
{
	int i, new_x;
	
	for (i = 0; i < LCD_X_RES; i++) {
		if (line_vector[i] == 1) {
			new_x = x + i;
			
			if (new_x >= 0 && new_x < LCD_X_RES) {
						
				set_pixel(new_x, y, color);
			}
		}
	}
	
	// tratam cazul cand incep jumatate goala
	// adaug doi pixeli la inceput
	if (x == 6) {
		set_pixel(0, 23, color);
		set_pixel(1, 23, color);
	}
}


/* printez un box pentru a marca nivelul pe care il doresc */
static void print_box(uint8_t x, uint8_t y, COLOR color)
{
	int i, j;
	
	for (i = 0; i < 15; i++) {
		for (j = 0; j < 13; j++) {
			if (box_matrix[i][j] == 1)
				set_pixel(x + i, y - j, color);
		}
	}
}


/* Selectarea nivelului: putem deplasa un box de pe 1 pe 2 si invers */
static void select_game(void)
{
	int i, exit, level = 1;
	
	// Afisez informatiile din pagina
	lcd_mystr_delay("SELECT", 15, 6);
	buzzer_reda(NOTA);
	
	lcd_mystr_delay("LEVEL", 32, 5);
	buzzer_reda(NOTA);
	
	delay(2);
	
	lcd_mystr("1", 55, 5);
	
	lcd_mystr("2", 55, 2);

	
	delay(4);
	
	buzzer_reda(NOTA);
	print_box(50, 43, BLACK);
	
	// Astept dupa OK
	while (1) {
	
		lcd_mystr("OK", 77, 4);		
		exit = 0;
		
		// Daca s-a apasat stanga sau dreapta voi muta box-ul
		// de pe 2 pe 1 sau invers, in functie de caz
		for (i = 0; i < 5; i++) {
			
			if (level == 2 && (PIND & B_LEFT) == 0) {
				level = 1;
				print_box(50, 19, WHITE);
				buzzer_reda(NOTA);
				delay(1);
				print_box(50, 43, BLACK);
			}
			else if (level == 1 && (PIND & B_RIGHT) == 0) {
				level = 2;
				print_box(50, 43, WHITE);
				buzzer_reda(NOTA);
				delay(1);
				print_box(50, 19, BLACK);
			}
			
			// verific si ok-ul
			if ((PIND & B_OK) == 0) {
				buzzer_reda(NOTA);
				exit = 1;
				break;
			}
			delay(1);
		}
		
		// conditie iesire
		if (exit) break;
		
		// stergem ok-ul (pentru a-l face intermitent)
		lcd_clear_area(4, 77, 84);
		lcd_clear_area(3, 77, 84);
		
		// Verific din nou daca s-au apasat butoane
		for (i = 0; i < 5; i++) {
			if (level == 2 && (PIND & B_LEFT) == 0) {
				level = 1;
				print_box(50, 19, WHITE);
				buzzer_reda(NOTA);
				delay(1);
				print_box(50, 43, BLACK);
			}
			else if (level == 1 && (PIND & B_RIGHT) == 0) {
				level = 2;
				print_box(50, 43, WHITE);
				buzzer_reda(NOTA);
				delay(1);
				print_box(50, 19, BLACK);
			}
			
			if ((PIND & B_OK) == 0) {
				buzzer_reda(NOTA);
				exit = 1;
				break;
			}
			delay(1);
		}
		if (exit) break;
	}
	
	lcd_clear();
	init_buffer();
	
	delay(2); 
	
	// Intram pe un nivel in functie de optiunea aleasa
	if (level == 1) {
		game_level1();
	}
	else {
		game_level2();
	}
}


/* Functie ce compara daca doua masini sunt pe aceeasi banda */
static int compare_position(int car_y, int player_y)
{
	if (player_y == LEFT && car_y == coord.left) {
		return 1;
	}
	else if(player_y == RIGHT && car_y == coord.right) {
		return 1;
	}
	else if(player_y == CENTER && car_y == coord.center) {
		return 1;
	}
	
	return 0;
}


/* Functie ce reda un ton */
static void play_game_over(void)
{
	int i;
	int time = 10000;
	for (i = 0; i < 5; i++) {
		buzzer_reda(NOTA);
		
		
		_delay_ms(time);
		PORTD &= ~BUZZER;
		time -= 1000;		
	}
}


/* jocul s-a terminat -> afisez starea jocului */
static void game_over(void)
{
	char buff[6];
	int exit, i;
	
	delay(10);
	
	// afisez text cu game over si in acelasi timp
	// pornesc si buzzer-ul
	lcd_clear();
	buzzer_reda(NOTA);
	lcd_mystr_delay("GAME", 25, 5);
	buzzer_reda(NOTA);
	lcd_mystr_delay("OVER", 45, 5);
	buzzer_reda(NOTA);
	
	play_game_over();
	
	delay(10);
	lcd_clear();
	
	// Afisez score-ul
	lcd_mystr_delay("Your", 15, 5);
	buzzer_reda(NOTA);
	lcd_mystr_delay("Score", 30, 5);
	buzzer_reda(NOTA);
	
	// centram scorul
	if (score < 10) {
		i = 3;
	}
	else if (score < 100) {
		i = 4;
	}
	else if (score < 1000) {
		i = 4;
	}
	else {
		i = 5;
	}
	sprintf (buff, "%d", score);
	lcd_mystr_delay(buff, 50, i);
	
	
	// astept pentru butonul ok
	while (1) {
		buzzer_reda(NOTA);
		lcd_mystr("Reset", 77, 5);
		
		
		exit = 0;
		
		for (i = 0; i < 5; i++) {
			if ((PIND & B_OK) == 0) {
				exit = 1;
				break;
			}
			delay(1);
		}
		if (exit) break;
		
		// intermitent
		for (i = 1; i < 6; i++)
			lcd_clear_area(i, 77, 84);
		
		
		for (i = 0; i < 5; i++) {
			if ((PIND & B_OK) == 0) {
				exit = 1;
				break;
			}
			delay(1);
		}
		
		if (exit) break;
	}

	delay(10);
}


/* Afisare cadru cand playerul pierde o viata */
static void print_321(void)
{
	char buff[2];
	
	delay(5);
	play_game_over();
	lcd_clear();

	// Afisez mesaj corespunzator
	lcd_mystr_delay("You", 20, 5);
	lcd_mystr_delay("lost a", 34, 6);
	lcd_mystr_delay("life", 48, 5);
	
	sprintf (buff, "(%d)", lives);
	lcd_mystr_delay(buff, 62, 4);
	
	delay(20);
	lcd_clear();
	
	// Revenire in joc in 3 pasi
	lcd_mystr_delay("Resume", 20, 6);
	lcd_mystr_delay("Game", 40, 5);
	
	lcd_mystr("3", 62, 3);
	buzzer_reda(NOTA);
	delay(10);
	lcd_clear_area(3, 62, 70);
	
	lcd_mystr("2", 62, 3);
	buzzer_reda(NOTA);
	delay(10);
	
	lcd_clear_area(3, 62, 70);
	lcd_mystr("1", 62, 3);
	buzzer_reda(NOTA);
	delay(10);
	
}


/******* NIVELUL 1 *******/
static void game_level1(void)
{
	int i;
	
	// setari level 1
	coord.left = 38;
	coord.right = 21;
	distance = 40;
	count = 0;
	score = 0;
	
	// Print intro level 1
	lcd_mystr_delay("Level", 25, 5);
	buzzer_reda(NOTA);
	lcd_mystr_delay("1", 42, 3);
	buzzer_reda(NOTA);
	
	delay(10);
	
	// print banda de loading
	int size1 = LCD_X_RES - 1;
	int size2 = LCD_X_RES - 2;
	int size3 = LCD_X_RES - 3;
	for(i = LCD_Y_RES - 1;i >= 0; i--)
    {
		set_pixel(size1, i, BLACK);
		set_pixel(size2, i, BLACK);
		set_pixel(size3, i, BLACK);
		
		if (i % 4 == 0)
			delay(5);
    }
	
	delay(1);
	
	// Pornim primul nivel al jocului 
	while (lives) {
		lcd_clear();
		init_buffer();
		
		level1();
		
		lives --;
		count = 0;
		
		// afisam cadrul pentru pierderea unei vieti
		if (lives > 0)
			print_321();
	}
	
	game_over();
}

/* Implementarea primului nivel */	
static void level1(void)
{
	int i;
	int random_position;
	int line_index = 0;
	int result;
	
	
	// linii exterior
	for(i = 0;i < LCD_X_RES; i++) {
		set_pixel(i, 40, BLACK);
		set_pixel(i, 6, BLACK);
		
		
    }
	
	// setarile initiale ale masinilor
	for (i = 0; i < 2; i++) {
		cars[i].x = -26;
		cars[i].y = coord.left;
		cars[i].active = 0;
	}
	
	// avem o masina inamica in circuit
	cars[0].active = 1;
	newcar = 0;
	
	// ii calculam pozitia random
	random_position = rand() % 100;
	
	if (random_position > 50) {
		cars[0].y = coord.right;
	}
	
	// desenam masina playerului
	player.y = coord.left; // in stanga
	print_car(player.x, player.y, BLACK);
	player_column = LEFT;
	
	// Afisez si linia ce delimiteaza benzile
	print_line(line_index, coord.right + 2, BLACK);
	
	delay(5);
	
	while(1) {
		// setam viteza pt cazul fast
		if (faster == 1)
			result = cars_move_level1(150);
		
		else
			result = cars_move_level1(speed);
		
		// daca jocul continua
		if (result == 1) {
			// vom creste din doi in doi si avem 4 cazuri de linie punctata cu
			// care incepem: intreaga, jumatate, jumatate white, total white
			print_line(line_index, coord.right + 2, WHITE);
			line_index = (line_index + 2) % 8;
			
			//printam linia de centru
			print_line(line_index, coord.right + 2, BLACK);
		}
		
		// jocul s-a incheiat
		else if (result == 2) {
			return;
		}
	}
}


/* Functie ce simuleaza miscarea masinilor in spatiu */
static int cars_move_level1(int speed)
{
	int random_position;
	int n = 2;
	int i;
	
	// verific ce masini sunt active si le desenez
	for (i = 0; i < n; i++) {
		if (cars[i].active == 1) {
			print_car(cars[i].x, cars[i].y, BLACK);
			
			// adaug o noua masina in caz ca s-a parcurs o distanta
			if (newcar == 0 && cars[i].x == distance) {
				newcar = 1;
			}
		}
	}
	
	// verific daca sunt pe modul fast sau nu
	if (faster == 0 && (PIND & B_OK) == 0) {
		faster = 1;
	}
	else if (faster == 1 && (PIND & B_OK) != 0) {
		faster = 0;
	}
	
	// verific daca adaug o masina noua;
	if (newcar == 1) {
		for (i = 0; i < n; i++) {
	
			// caut o masina inactiva si o activez
			if (cars[i].active == 0) {
				cars[i].active = 1;
				newcar = 0;
				
				// ii setez pozitia random in joc
				random_position = rand() % 100;

				if (random_position > 50) {
					cars[i].y = 21;
				}
				else {
					cars[i].y = 38;
				}
				
				// o afisez la final
				print_car(cars[i].x, cars[i].y, BLACK);
				
				break;
			}
		}
	}
	
	// Verific butoanele de left/right si mut playerul conform acestora
	if (player_column == LEFT && (PIND & B_RIGHT) == 0) {
	
		// sterg vechea pozitie
		player_column = RIGHT;
		print_car(player.x, player.y, WHITE);
		
		// sunet buton 
		buzzer_reda(NOTA);
		
		// ma mut in dreapta
		player.y = coord.right;
		
		_delay_ms(speed);
		// afisez noua pozitie
		print_car(player.x, player.y, BLACK);
	}
	
	else if (player_column == RIGHT && (PIND & B_LEFT) == 0) {
		
		// sterg vechea pozitie
		player_column = LEFT;
		print_car(player.x, player.y, WHITE);
		
		// sunet buton
		buzzer_reda(NOTA);
		
		// ma mut in stanga
		player.y = coord.left;
		
		_delay_ms(speed);
		// afisez noua pozitie a playerului
		print_car(player.x, player.y, BLACK);
	}
	
	// mentinem delay-ul in caz ca nu se intra in conditiile de mai sus
	// si sa nu avem o inconsecventa
	else {
		_delay_ms(speed);
	}
	
	// Verificare coliziuni
	for (i = 0; i < n; i++) {
		// ma uit daca masina e in drept cu mine			
		if (cars[i].x > 33) {
			// jocul s-a terminat (viata --)
			if(compare_position(cars[i].y, player_column) == 1)
				return 2;
		}
	}

	// intarzii afisarea masinilor
	_delay_ms(speed);
	
	// Stergem masinile frame-ul curent, si avansam masinile inamice
	// pentru noul frame
	for (i = 0; i < n; i++) {
		
		// stergem masina de la pasul curent
		if (cars[i].active == 1) {
			print_car(cars[i].x, cars[i].y, WHITE);
			
			// deplasam masinia inspre player
			cars[i].x += 2;
			
			// verific daca masina a ajuns in capat
			if (cars[i].x > 81) {
				cars[i].x = -26;
				cars[i].active = 0;
				score ++;
				
				// setez viteza in functie de score
				if (speed < 400) {
					speed = 400;
				}
				else {
					speed = 800 - score * 20;
				}
				
				// scad si distanta dintre masini de la 40 la 32 (in fct
				// de scorul adunat
				if (score != 0 && score % 10 == 0 && distance > 30) {
					distance -= 2;
				}
			}
		}

	}
	// cadru realizat cu succes
	return 1;
}


/******* NIVELUL 2 *******/
static void game_level2(void)
{
	int i;
	
	// setati pentru level 2
	coord.left = 47;
	coord.center = 30;
	coord.right = 13;
	distance = 4;
	speed = 800;
	count = 0;
	score = 0;
	
	// print intro level 2
	lcd_mystr_delay("Level", 25, 5);
	buzzer_reda(NOTA);
	lcd_mystr_delay("2", 42, 3);
	buzzer_reda(NOTA);
	
	
	delay(10);
	
	// prind banda de loading
	int size1 = LCD_X_RES - 1;
	int size2 = LCD_X_RES - 2;
	int size3 = LCD_X_RES - 3;
	for(i = LCD_Y_RES - 1;i >= 0; i--)
    {
		set_pixel(size1, i, BLACK);
		set_pixel(size2, i, BLACK);
		set_pixel(size3, i, BLACK);
		
		if (i % 4 == 0)
			delay(5);
    }
	
	delay(1);
	
	// pornim al doilea nivel al jocului
	while (lives) {
		lcd_clear();
		init_buffer();
		
		level2();
		
		lives --;
		count = 0;
		
		// afisam cadrul pentru pierderea unei vieti
		if (lives > 0)
			print_321();
	}
	
	game_over();	
}


/* Implementarea celui de al doilea nivel */
static void level2(void)
{
	int i;
	int random_position;
	int line_index = 0;
	int result;
	
	// setarile initiale ale masinilor
	for (i = 0; i < 4; i++) {
		cars[i].x = -26;
		cars[i].y = coord.left;
		cars[i].active = 0;
	}
	
	// avem o masina inamica in circuit
	cars[0].active = 1;
	newcar = 0;
	
	// ii calculam pozitia random
	random_position = rand() % 100;
	
	if (random_position > 66) {
		cars[0].y = coord.right;
	}
	else if (random_position > 33) {
		cars[0].y = coord.center;
	}
	
	// setez numarul masinii (in circuit vor
	// exista maxim 4 masini in acelasi timp)
	cars[0].num = count;
	count++;
	
	// desenam masina playerului in centru
	player.y = coord.center; 
	print_car(player.x, player.y, BLACK);
	player_column = CENTER;
	
	// printez si liniile ce delimiteaza benzile
	print_line(line_index, coord.center+ 2, BLACK);
	print_line(line_index, coord.right + 2, BLACK);
	
	delay(5);
	
	while(1) {
		// setam viteza daca suntem pe cazul fast
		if (faster == 1)
			result = cars_move_level2(100);
		
		else
			result = cars_move_level2(speed);
		
		// daca jocul continua
		if (result == 1) {
			// vom creste din doi in doi si avem 4 cazuri de linie punctata cu
			// care incepem: intreaga, jumatate, jumatate white, total white
			print_line(line_index, coord.center+ 2, WHITE);
			print_line(line_index, coord.right + 2, WHITE);
			line_index = (line_index + 2) % 8;
			
			//printam liniile delimitatoare
			print_line(line_index, coord.center+ 2, BLACK);
			print_line(line_index, coord.right + 2, BLACK);
		}
		
		// jocul s-a incheiat
		else if (result == 2) {
			return;
		}
	}
}


/* Functie ce simuleaza miscarea masinilor in spatiu */
static int cars_move_level2(int speed)
{
	// Avem 4 masini la un moment dat in circuit
	int n = 4;
	int i;
	
	// verific ce masini sunt active si le desenez
	for (i = 0; i < n; i++) {
		if (cars[i].active == 1) {
			print_car(cars[i].x, cars[i].y, BLACK);
			
			// adaug o noua masina in caz ca s-a parcurs o distanta
			if (newcar == 0 && cars[i].x == distance) {
				newcar = 1;
			}
		}
	}
	
	// verific daca sunt pe fast sau nu
	if (faster == 0 && (PIND & B_OK) == 0) {
		faster = 1;
	}
	else if (faster == 1 && (PIND & B_OK) != 0) {
		faster = 0;
	}
	
	// verific daca adaug o masina noua;
	if (newcar == 1) {
		for (i = 0; i < n; i++) {
	
			// caut o masina inactiva
			if (cars[i].active == 0) {
				
				// ii setez pozitia random in joc
				cars[i].y = random_car_position();
			
				// o activez si o adaug in circuit
				cars[i].active = 1;
				cars[i].num = count;
				
				count = (count + 1) % 4;
				newcar = 0;
				
				// afisare masina
				print_car(cars[i].x, cars[i].y, BLACK);
				
				break;
			}
		}
	}
	
	// Verific butoanele de left/right si mut playerul conform acestora
	// stanga -> centru
	if (player_column == LEFT && (PIND & B_RIGHT) == 0) {
	
		// sterg vechea pozitie
		player_column = CENTER;
		print_car(player.x, player.y, WHITE);
		
		// redau sunetul pt buton
		buzzer_reda(NOTA);
		
		// ma mut pe centru
		player.y = coord.center;
		
		_delay_ms(speed);
		// afisez noua pozitie
		print_car(player.x, player.y, BLACK);
	}
	
	// centru -> stanga
	else if (player_column == CENTER && (PIND & B_LEFT) == 0) {
	
		// sterg vechea pozitie
		player_column = LEFT;
		print_car(player.x, player.y, WHITE);
		
		// sunet buton
		buzzer_reda(NOTA);
		
		// ma mut in stanga
		player.y = coord.left;
		
		_delay_ms(speed);
		// afisez noua pozitie
		print_car(player.x, player.y, BLACK);
	}
	
	// centru >-> dreapta
	else if (player_column == CENTER && (PIND & B_RIGHT) == 0) {
	
		// sterg vechea pozitie
		player_column = RIGHT;
		print_car(player.x, player.y, WHITE);
		
		// sunet buton
		buzzer_reda(NOTA);
		
		// ma mut in stanga
		player.y = coord.right;
		
		_delay_ms(speed);
		// afisez noua pozitie
		print_car(player.x, player.y, BLACK);
	}
	
	// dreapta -> centru
	else if (player_column == RIGHT && (PIND & B_LEFT) == 0) {
		
		// sterg vechea pozitie
		player_column = CENTER;
		print_car(player.x, player.y, WHITE);
		
		// sunet buton
		buzzer_reda(NOTA);
		
		// ma mut in stanga
		player.y = coord.center;
		
		_delay_ms(speed);
		// afisez noua pozitie
		print_car(player.x, player.y, BLACK);
	}
	
	// mentinem delay-ul in caz ca nu se intra in conditiile de mai sus
	// si sa nu avem o inconsecventa
	else {
		_delay_ms(speed);
	}
	
	// Verificare coliziuni
	for (i = 0; i < n; i++) {
		// ma uit daca masina e in drept cu mine			
		if (cars[i].x > 33) {
		
			// jocul s-a terminat
			if(compare_position(cars[i].y, player_column) == 1)
				return 2;
		}
	}

	/// intarzii afisarea masinilor
	_delay_ms(speed);
	
	// Stergem masinile frame-ul curent, si avansam masinile inamice
	// pentru noul frame
	for (i = 0; i < n; i++) {
		
		// stergem masina de la pasul curent
		if (cars[i].active == 1) {
			print_car(cars[i].x, cars[i].y, WHITE);
			
			// deplasam masina catre player
			cars[i].x += 2;
			
			// verific daca masina a ajuns in capat
			if (cars[i].x > 81) {
				cars[i].x = -26;
				cars[i].active = 0;
				score ++;
				
				// setez viteza in functie de score
				if (speed < 400) {
					speed = 400;
				}
				else {
					speed = 600 - score * 20;
				}
			}
		}
	}
	
	// cadru realizat cu succes
	return 1;
}


/* Setam pozitia random - > Ideea este sa nu inchidem circuitul,
  adica sa nu am o diagonala de 3 masini */
static int random_car_position(void)
{
	int i = 0;
	// calculez poztia random
	int random_position = rand() % 100;
	
	// ma uit la ultimele 2 masini adaugate
	int car1, car2;
	
	int sum = 0;
	
	// verific daca e doar o masinia in circuit
	for (i = 0; i < 4; i++) {
		sum += cars[i].active;
	}
	
	// am doar o masina activa -> afisez pe orice banda
	if (sum < 2) {
	
		if (random_position > 66) {
			return coord.right;
		}
		
		if (random_position > 33) {
			return coord.center;
		}
		
		return coord.left;
	}
	
	// gasim ultimele doua masini adaugate in circuit
	for (i = 0; i < 4; i++) {
		if (cars[i].active == 1) {
			if (cars[i].num == ((count + 3) % 4)) {
				car2 = cars[i].y;
			}
			if(cars[i].num == ((count + 2) % 4)) {
				car1 = cars[i].y;
			}	
		}
	}
	
	/* daca formez o diagonala -> aleg dintre 2 pozitii (cea de centru,
	   sau cea a masinei din coltul de jos al diagonalei, astfel sa exclud
	   situatia formarii unei diagonale de 3 masini) */
	if ((car1 == coord.left && car2 == coord.center) ||
		(car1 == coord.right && car2 == coord.center)) {

		// returnez pozitia masinii din coltul diagonalei
		if (random_position > 50) {
			return car1;
		}
		
		// returnez poz de centru
		return car2;
	}
	
	/* daca sunt doua masini pe centru si exista si a treia pe 
	   un colt ( a treia fiind cea mai aproape de player) atunci
	   va trebui sa pun ori pe colt, ori pe poz celei de a treia masini */
	
	if (car1 == coord.center && car1 == car2 && sum > 2) {
	
		// caut a treia masina (cea mai aproape de player)
		for (i = 0; i < 4; i++) {
			if (cars[i].active == 1 &&
				(cars[i].num == ((count + 1) % 4))) {
				car2 = cars[i].y;
				break;
			}	
		}
		
		// daca pozitia ei este tot in centru -> trebuie sa adaug
		// in centru sau opus pozitiei playerului
		if (car2 == coord.center) {
			if (player_column == LEFT) {
				car2 = coord.right;
			}
			else {
				car2 = coord.left;
			}
		}
		
		// returnez pozitia finala (centru sau poz masinei 3)
		if (random_position > 50) {
			return car1;
		}
		
		return car2;

	}
	
	// altfel, returnez orice pozitie
	if (random_position > 66) {
		return coord.right;
	}
	
	if (random_position > 33) {
		return coord.center;
	}
	
	return coord.left;
}
