
/*
	MARCU DENIS-GEORGE
	332CA
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include "lcd.h"

#define B_LEFT			(1 << PD0)
#define B_RIGHT			(1 << PD1)
#define B_OK		 	(1 << PD4)
#define BUZZER			(1 << PD5)

#define LEFT			0
#define CENTER			2
#define RIGHT			1

#define BUFF_SIZE_X      (LCD_X_RES)
#define BUFF_SIZE_Y      (LCD_Y_RES / 8)

#define NOTA			440

/* matricea corespunzatoare desenarii masinii */
char car_matrix [26][14] = {
	{0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0},
	{0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0},
	{0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0},
	{0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0},
	{0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0},
	{0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0},
	{0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0},
	{0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0},
	{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0},
	{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0},
	{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0},
	{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0},
	{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0},
	{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0},
	{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0},
	{1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1},
	{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
	{1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1},
	{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0},
	{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0},
	{0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0},
	{0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0},
	{0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0},
	{0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0}
};

/* vectorul pentru desenarea liniei delimitatoare */
char line_vector[84] =
{
	1, 1, 1, 1,
	0, 0, 0, 0,
	1, 1, 1, 1,
	0, 0, 0, 0,
	1, 1, 1, 1,
	0, 0, 0, 0,
	1, 1, 1, 1,
	0, 0, 0, 0,
	1, 1, 1, 1,
	0, 0, 0, 0,
	1, 1, 1, 1,
	0, 0, 0, 0,
	1, 1, 1, 1,
	0, 0, 0, 0,
	1, 1, 1, 1,
	0, 0, 0, 0,
	1, 1, 1, 1,
	0, 0, 0, 0,
	1, 1, 1, 1,
	0, 0, 0, 0,
	1, 1, 1, 1
};

/* matrice corespondenta unui box pt selectie */
char box_matrix [15][13] = {
	{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
	{1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1},
	{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}
};

/* litere mari: C, A, R, E */
char letter_matrix [48][7] = {
	// **** C ****
	{0, 1, 1, 1, 1, 1, 0},
	{1, 1, 1, 1, 1, 1, 1},
	{1, 1, 0, 0, 0, 1, 1},
	{1, 1, 0, 0, 0, 0, 0},
	{1, 1, 0, 0, 0, 0, 0},
	{1, 1, 0, 0, 0, 0, 0},
	{1, 1, 0, 0, 0, 0, 0},
	{1, 1, 0, 0, 0, 0, 0},
	{1, 1, 0, 0, 0, 0, 0},
	{1, 1, 0, 0, 0, 1, 1},
	{1, 1, 1, 1, 1, 1, 1},
	{0, 1, 1, 1, 1, 1, 0},
	
	// **** A ****
	{0, 1, 1, 1, 1, 1, 0},
	{1, 1, 1, 1, 1, 1, 1},
	{1, 1, 0, 0, 0, 1, 1},
	{1, 1, 0, 0, 0, 1, 1},
	{1, 1, 0, 0, 0, 1, 1},
	{1, 1, 0, 0, 0, 1, 1},
	{1, 1, 1, 1, 1, 1, 1},
	{1, 1, 1, 1, 1, 1, 1},
	{1, 1, 0, 0, 0, 1, 1},
	{1, 1, 0, 0, 0, 1, 1},
	{1, 1, 0, 0, 0, 1, 1},
	{1, 1, 0, 0, 0, 1, 1},
	
	// **** R ****
	{1, 1, 1, 1, 1, 1, 0},
	{1, 1, 1, 1, 1, 1, 1},
	{1, 1, 0, 0, 0, 1, 1},
	{1, 1, 0, 0, 0, 1, 1},
	{1, 1, 0, 0, 0, 1, 1},
	{1, 1, 1, 1, 1, 1, 1},
	{1, 1, 1, 1, 1, 1, 0},
	{1, 1, 0, 1, 1, 0, 0},
	{1, 1, 0, 0, 1, 1, 0},
	{1, 1, 0, 0, 1, 1, 0},
	{1, 1, 0, 0, 0, 1, 1},
	{1, 1, 0, 0, 0, 1, 1},
	
	// **** E ****
	{1, 1, 1, 1, 1, 1, 1},
	{1, 1, 1, 1, 1, 1, 1},
	{1, 1, 0, 0, 0, 0, 0},
	{1, 1, 0, 0, 0, 0, 0},
	{1, 1, 0, 0, 0, 0, 0},
	{1, 1, 1, 1, 1, 0, 0},
	{1, 1, 1, 1, 1, 0, 0},
	{1, 1, 0, 0, 0, 0, 0},
	{1, 1, 0, 0, 0, 0, 0},
	{1, 1, 0, 0, 0, 0, 0},
	{1, 1, 1, 1, 1, 1, 1},
	{1, 1, 1, 1, 1, 1, 1}
};

/* structura pentru masina */
typedef struct {
    int x, y;
	int active;		// daca masina exista
	int num;		// contor pt numarul masinii
} CAR;

/* coordonatele y ale unei masini */
typedef struct {
    int left, right, center;
} COORD;

/* coordonatele masinei player */
typedef struct {
    int x, y;
} PLAYER;

typedef enum {BLACK, WHITE} COLOR;

int lives = 3;
static int score = 0;

static CAR cars[4];
PLAYER player;
static COORD coord;

static int newcar = 0;
static int speed = 800;
static int faster = 0;
static int player_column = LEFT;
static int distance = 40;
static int count = 0;

/* matricea buffer a lcd-ului folosita la setarea pixelului */
static uint8_t lcd_buffer[BUFF_SIZE_Y][BUFF_SIZE_X];

/* initializare butoane */
static void init_buttoms(void);

/* initializarea bufferul pentru lcd */
static void init_buffer(void);

/* initializarea buzzer-ului */
static void init_buzzer(void);

/* functie ce reda o nota */
static void buzzer_reda(int nota);

/* functie de setare a unui pixel */
static void set_pixel(uint8_t x, uint8_t y, COLOR color);

/* functie de delay */
static void delay(uint8_t n);

/* functie ce afiseaza deschiderea unui program */
static void openning_program(void);

/* functie de initializare a jocului */
static void init_game(void);

/* selectarea nivelului */
static void select_game(void);

/* functie ce printeaza o litera mare (C/A/R/E) */
static void print_letter(uint8_t x, uint8_t y,
    COLOR color, uint8_t index);

/* afisam cuvantul CAR */
static void lcd_car(uint8_t x, uint8_t y, COLOR color);

/*afisam cuvantul RACE */
static void lcd_race(uint8_t x, uint8_t y, COLOR color);

/* printez masina */
static void print_car(int x, int y, COLOR color);

/* printez masina invers */
static void print_car_reverse(int x, int y, COLOR color);

/* printez linia de delimitare */
static void print_line(int x, int y, COLOR color);

/* printez un box pentru a marca nivelul pe care il doresc */
static void print_box(uint8_t x, uint8_t y, COLOR color);

/* Functie ce compara daca doua masini sunt pe aceeasi banda */
static int compare_position(int car_y, int player_y);

/* Functie ce porneste un ton la sfarsitul jocului */
static void play_game_over(void);

/* Afisare GAME OVER */
static void game_over(void);

/* Afisez acest cadru la pierderea unei vieti */
static void print_321(void);

/* functii lvl 1 */
static int cars_move_level1(int speed);
static void level1(void);
static void game_level1(void);

/* functii lvl 2 */
static int random_car_position(void);
static int cars_move_level2(int speed);
static void level2(void);
static void game_level2(void);
